#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// parameters of the model
const int n = 20000;
const int d = 100;
const float s = 1.5;
const float a = 0.8;

int v [n][n];
int vert_to_rank [n];
int rank_to_vert [n+1];
int discovered [n];
int init_rank [n];
int age [n];

float distr_init [n];
float distr_rank [n];

int main()
{
  int t=0;
  int i;
  int nr_to_discover = n;

  for (i=0 ; i<n ; i++) 
    {
      vert_to_rank[i]=i+1;
      rank_to_vert[i+1]=i;
      discovered[i]=0;
      init_rank[i]=-7;
      age[i]=-7;
    }

  for (i=0 ; i<n/2 ; i++)
    distr_init[i] = 0.5*exp(s*log(2.0*(i+1)/n));
  for (i=n/2 ; i<n-1 ; i++)
    distr_init[i] = 1.0 - 0.5*exp(s*log(2.0*(n-i-1)/n));
  distr_init[n-1] = 1.0;

  distr_rank[0] = 1.0;
  for (i=1 ; i<n ; i++)
    distr_rank[i] = distr_rank[i-1] + exp(-a*log(i+1.0));
  for(i=0 ; i<n ; i++)
    distr_rank[i] /= distr_rank[n-1];

  while(nr_to_discover != 0)
    {
      t++;
      // removing one vertex uniformly at random
      int to_remove = rand() % n;
      if (discovered[to_remove] == 0)
	{
	  discovered[to_remove]=1;
	  nr_to_discover--;
	}
      for (i=0 ; i<n ; i++) v[to_remove][i]=v[i][to_remove]=0;

      for (i=vert_to_rank[to_remove]+1; i<=n ; i++)
	{
	  vert_to_rank[ rank_to_vert[i] ]--;
	  rank_to_vert[i-1] = rank_to_vert[i];
	}
      rank_to_vert[n] = -7;

      // generating new rank
      float p = 1.0*rand()/RAND_MAX;
      int new_rank=0;
      while (distr_init[new_rank] <= p) new_rank++;
      new_rank++;

      for (i=n-1 ; i>=new_rank ; i--)
	{
	  vert_to_rank[ rank_to_vert[i] ]++;
	  rank_to_vert[i+1] = rank_to_vert[i];
	}
      rank_to_vert[new_rank]=to_remove;
      vert_to_rank[to_remove]=new_rank;
      init_rank[to_remove]=new_rank;

      // generating new edges
      for (i=0 ; i<d ; i++)
	{
	  float p = 1.0*rand()/RAND_MAX;
	  new_rank=0;
	  while(distr_rank[new_rank] <= p) new_rank++;
	  new_rank++;

	  v[to_remove][ rank_to_vert[new_rank] ] ++ ;   
	}
      age[to_remove]=t;
    }

  // degree distribution
  int deg [n];
  for (i=0 ; i<n ; i++)
    {
      deg[i]=0;
      for (int j=0 ; j<n ; j++)
	deg[i] += v[j][i];
    }

  printf("\n\n DEGREE DISTRIBUTION \n\n");
  int deg_dist [d*n];
  for (i=0 ; i< d*n ; i++) deg_dist[i]=0;
  for (i=0 ; i<n ; i++)
    deg_dist[ deg[i] ]++;
  for (i=d*n-2 ; i >= 0 ; i--)
    deg_dist[i] += deg_dist[i+1];

  for (i=1 ; i<d*n-1 ; i++)
    if (deg_dist[i] != deg_dist[i+1]) printf("%f %f \n", 1.0*i, 1.0*deg_dist[i]);

  printf("\n\n AGE_RANK vs. DEGREE \n\n");
  // predicting age (trivial approach: age vs. degree)
  for (i=0 ; i<n ; i++)
    if (deg[i] > 0.5*d)
      {
	int age_r=1;
	for (int j=0 ; j<n ; j++) 
	  if (age[j] < age[i]) age_r++;
	
	// float r = 0.5*n*exp(log(exp(-(1-s)*log(deg[i]*exp((1-a)*log(2))/d)/a)+1)/(1-s));
	printf("%f %f\n", 1.0*age_r/n, 1.0*deg[i]/d);
      }

  printf("\n\n 1ST NEIGHBOURHOOD \n\n");
  // predicting initial rank -- 1st neighbourhood
  float coef[n];
  for (i=0 ; i<n ; i++)
    {
      coef[i]=0;
      if (deg[i] > 0)
	{
	  int sum_deg=0;
	  int age_r=1;
	  for (int j=0 ; j<n ; j++)
	    {
	      if (v[j][i]) sum_deg+=deg[j];
	      if (age[j] < age[i]) age_r++;
	    }
	  coef[i]=1.0*sum_deg/(d*deg[i]);
	  if (deg[i] > 0.5*d)
	    printf("%f %f \n", 1.0*age_r/n, coef[i]); 
	}
    }

  printf("\n\n 2ND NEIGHBOURHOOD \n\n");
  // predicting initial rank -- 2nd neighbourhood
  float coef2[n];
  for (i=0 ; i<n ; i++)
    {
      coef2[i]=0;
      if (deg[i] > 0)
	{
	  float sum_coef=0;
	  int age_r=1;
	  for (int j=0 ; j<n ; j++)
	    {
	      if (v[j][i]) sum_coef+=coef[j];
	      if (age[j] < age[i]) age_r++;
	    }
	  
	  coef2[i]=1.0*sum_coef/deg[i];
	  if (deg[i] > 0.5*d)
	    printf("%f %f \n", 1.0*age_r/n, coef2[i]);
      }
    }

  printf("\n\n 3RD NEIGHBOURHOOD \n\n");
  // predicting initial rank -- 3rd neighbourhood
  float coef3[n];
  for (i=0 ; i<n ; i++)
    {
      coef3[i]=0;
      if (deg[i] > 0)
	{
	  float sum_coef=0;
	  int age_r=1;
	  for (int j=0 ; j<n ; j++)
	    {
	      if (v[j][i]) sum_coef+=coef2[j];
	      if (age[j] < age[i]) age_r++;
	    }
	  
	  coef3[i]=1.0*sum_coef/deg[i];
	  if (deg[i] > 0.5*d)
	    printf("%f %f \n", 1.0*age_r/n, coef3[i]);
	}
    }

  return 0;
}
